#!/bin/sh

# This script is designed to create an ext2ci module based on the ext2
# module which is case-insensitive. Fortunately, the ext2 filesystem designer
# made this extremely easy -- just change the very first function in namei.c.
# The hard part is making a copy of the module so that you can select which
# filesystems to mount case-insensitive and which to mount normally. The nice
# part about using the seperate-module approach is that there is no
# performance penalty for normal filesystems.

# INSTRUCTIONS: 
# in the linux subtree, add a directory called fs/ext2ci. Copy
# this file into that directory and run it. 
# Reconfigure the kernel, adding ext2ci. Rebuild. Now, mount the desired ext2
# filesystem as type "ext2ci".


cp ../ext2/*.c .
cp ../../include/linux/ext2*.h .
cp ../ext2/Makefile .
rm *.o 2>>/dev/null
rm -f ../filesystems.a

H=`ls ext2*.h | grep -v ext2ci`
for i in $H ;do
  mv $i ext2ci${i##ext2}
done

for i in *.c *.h Makefile ;do
  sed '
	s/ext2/ext2ci/g
	s/ext2ci_group_desc/ext2_group_desc/g
	s/ext2ci_super_block/ext2_super_block/g
	s/<linux\/ext2ci_fs.h>/"ext2ci_fs.h"/g
	s/ext2ci_debug/ext2_debug/g
	s/ext2ci_dir_entry/ext2_dir_entry/g
	s/ext2ci_sb/ext2_sb/g
	s/ext2ci_i\./ext2_i\./g
	# s/ext2ci_inode/ext2_inode/g
	s/ext2ci_test_bit/ext2_test_bit/g
	s/ext2ci_clear_bit/ext2_clear_bit/g
	s/ext2ci_set_bit/ext2_set_bit/g
	s/ext2ci_find_next_zero_bit/ext2_find_next_zero_bit/g
	s/ext2ci_find_first_zero_bit/ext2_find_first_zero_bit/g
	s/EXT2CI/EXT2/g
	s/CONFIG_EXT2_FS/CONFIG_EXT2CI_FS/g
	' ${i} > ${i}.2
  mv ${i}.2 ${i}
  echo ${i}
done

echo "Lowercase dir.c..."
LINE=`cat -n dir.c | grep memcmp | awk '{ print $1 }'`
BEFORE=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 1`
DATA=`tail +${LINE} dir.c | head -1`

head -${BEFORE} dir.c > dir.c.2
echo "/* ${DATA} */" >> dir.c.2
cat << EOM >> dir.c.2
  na = (unsigned char*) name;
  nb = (unsigned char*) de->name;
  for (; len>0; na++,nb++,len--)
    if (ext2ci_lowercasemap[(int) *na] != ext2ci_lowercasemap[(int) *nb])
	return 0;
  return 1;
EOM
tail +${AFTER} dir.c >> dir.c.2

LINE=`cat -n dir.c.2 | grep 'static.*int ext2ci_match' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE} + 2`
AFTER=`expr ${LINE} + 3`

head -${INSERTAT} dir.c.2 > dir.c
echo "	unsigned char *na, *nb;" >> dir.c
tail +${AFTER} dir.c.2 >> dir.c

LINE=`cat -n dir.c | grep 'static.*int ext2ci_match' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 0`

head -${INSERTAT} dir.c > dir.c.2
echo "extern const char *ext2ci_lowercasemap;" >> dir.c.2
tail +${AFTER} dir.c >> dir.c.2

mv dir.c.2 dir.c

echo "Lowercase namei.c"

LINE=`cat -n namei.c | grep ' \* Methods themselves.' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE} + 1`
AFTER=`expr ${LINE} + 2`

head -${INSERTAT} namei.c > namei.c.2
cat ext2ci-sed-2.4.data.1 >> namei.c.2
tail +${AFTER} namei.c >> namei.c.2

LINE=`cat -n namei.c.2 | grep 'ino = ext2ci_inode_by_name(dir, dentry);' | \
	 awk '{ print $1 }'`
INSERTAT=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 0`

head -${INSERTAT} namei.c.2 > namei.c
echo "	dentry->d_op = &ext2ci_dentry_operations;" >> namei.c
tail +${AFTER} namei.c.2 >> namei.c


echo "Lowercase super.c"

LINE=`cat -n super.c | grep 'struct super_block . ext2ci_read_super' | \
	 awk '{ print $1 }'`
INSERTAT=`expr ${LINE} - 1`
AFTER=`expr ${LINE} + 0`

head -${INSERTAT} super.c > super.c.2
echo "extern struct dentry_operations ext2ci_dentry_operations;" >> super.c.2
tail +${AFTER} super.c >> super.c.2

LINE=`cat -n super.c.2 | \
	grep 'ext2ci_setup_super (sb, es, sb->s_flags . MS_RDONLY);' | \
	 awk '{ print $1 }'`
INSERTAT=`expr ${LINE} + 0`
AFTER=`expr ${LINE} + 1`

head -${INSERTAT} super.c.2 > super.c
echo "	sb->s_root->d_op = &ext2ci_dentry_operations;" >> super.c
tail +${AFTER} super.c.2 >> super.c

echo "FIX ../Config.in"
# Fix fs/Config.in so that ext2ci can be included in the kernel
grep -v 'CONFIG_EXT2CI_FS' ../Config.in > ../Config.in.2
LINE=`cat -n ../Config.in.2 | grep 'CONFIG_EXT2_FS' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE}`
AFTER=`expr ${LINE} + 1`

head -${INSERTAT} ../Config.in.2 > ../Config.in.3
echo "tristate 'Case Insensitive Ext2 fs support' CONFIG_EXT2CI_FS" >> ../Config.in.3
tail +${AFTER} ../Config.in.2 >> ../Config.in.3

mv ../Config.in.3 ../Config.in


echo "FIX ../Makefile"
# Fix fs/Makefile so that ext2ci can be included in the kernel
grep -v 'CONFIG_EXT2CI_FS' ../Makefile > ../Makefile.2
LINE=`cat -n ../Makefile.2 | grep 'CONFIG_EXT2_FS' | awk '{ print $1 }'`
INSERTAT=`expr ${LINE}`
AFTER=`expr ${LINE} + 1`

head -${INSERTAT} ../Makefile.2 > ../Makefile.3
echo "subdir-\$(CONFIG_EXT2CI_FS)	+= ext2ci" >> ../Makefile.3
tail +${AFTER} ../Makefile.2 >> ../Makefile.3

mv ../Makefile.3 ../Makefile




